@@ -9,7 +9,7 @@ onedash = set(zs.split())
 # grep volflag= __main__.py | sed -r 's/.*volflag=//;s/\).*//' | sort | uniq | while IFS= read -r x; do grep -E "\"$x(=[^ \"]+)?\": \"" cfg.py || printf '%s\n' "$x"; done
 
 
-def vf_bmap()   :
+def vf_bmap() -> dict[str, str]:
     """argv-to-volflag: simple bools"""
     ret = {
         "dav_auth": "davauth",
@@ -30,6 +30,7 @@ def vf_bmap()   :
     }
     for k in (
         "dedup",
+        "dlni",
         "dotsrch",
         "e2d",
         "e2ds",
@@ -79,7 +80,7 @@ def vf_bmap()   :
     return ret
 
 
-def vf_vmap()   :
+def vf_vmap() -> dict[str, str]:
     """argv-to-volflag: simple values"""
     ret = {
         "ac_convt": "aconvt",
@@ -150,7 +151,7 @@ def vf_vmap()   :
     return ret
 
 
-def vf_cmap()   :
+def vf_cmap() -> dict[str, str]:
     """argv-to-volflag: complex/lists"""
     ret = {}
     for k in (
@@ -318,6 +319,7 @@ flagcats = {
         "hsortn": "number of sort-rules to add to media URLs",
         "ufavico=URL": "per-volume favicon (.ico/png/gif/svg)",
         "unlist": "dont list files matching REGEX",
+        "dlni": "force-download (no-inline) files on click",
         "html_head=TXT": "includes TXT in the <head>, or @PATH for file at PATH",
         "html_head_s=TXT": "additional static text in the html <head>",
         "tcolor=#fc0": "theme color (a hint for webbrowsers, discord, etc.)",
@@ -407,3 +409,9 @@ flagcats = {
 
 flagdescs = {k.split("=")[0]: v for tab in flagcats.values() for k, v in tab.items()}
 
+
+if True:  # so it gets removed in release-builds
+    for fun in [vf_bmap, vf_cmap, vf_vmap]:
+        for k in fun().values():
+            if k not in flagdescs:
+                raise Exception("undocumented volflag: " + k)
