@@ -39,24 +39,28 @@ from .util import (
 if TYPE_CHECKING:
     from .svchub import SvcHub
 
+if True:  # pylint: disable=using-constant-test
+    import typing
+    from typing import Any, Optional, Union
+
 if PY2:
     range = xrange  # type: ignore
 
 
 class FSE(FilesystemError):
-    def __init__(self, msg , severity  = 0)  :
+    def __init__(self, msg: str, severity: int = 0) -> None:
         super(FilesystemError, self).__init__(msg)
         self.severity = severity
 
 
 class FtpAuth(DummyAuthorizer):
-    def __init__(self, hub )  :
+    def __init__(self, hub: "SvcHub") -> None:
         super(FtpAuth, self).__init__()
         self.hub = hub
 
     def validate_authentication(
-        self, username , password , handler 
-    )  :
+        self, username: str, password: str, handler: Any
+    ) -> None:
         handler.username = "{}:{}".format(username, password)
         handler.uname = "*"
 
@@ -115,33 +119,33 @@ class FtpAuth(DummyAuthorizer):
 
         handler.uname = handler.username = uname
 
-    def get_home_dir(self, username )  :
+    def get_home_dir(self, username: str) -> str:
         return "/"
 
-    def has_user(self, username )  :
+    def has_user(self, username: str) -> bool:
         asrv = self.hub.asrv
         return username in asrv.acct or username in asrv.iacct
 
-    def has_perm(self, username , perm , path  = None)  :
+    def has_perm(self, username: str, perm: int, path: Optional[str] = None) -> bool:
         return True  # handled at filesystem layer
 
-    def get_perms(self, username )  :
+    def get_perms(self, username: str) -> str:
         return "elradfmwMT"
 
-    def get_msg_login(self, username )  :
+    def get_msg_login(self, username: str) -> str:
         return "sup {}".format(username)
 
-    def get_msg_quit(self, username )  :
+    def get_msg_quit(self, username: str) -> str:
         return "cya"
 
 
 class FtpFs(AbstractedFS):
     def __init__(
-        self, root , cmd_channel 
-    )  :  # pylint: disable=super-init-not-called
+        self, root: str, cmd_channel: Any
+    ) -> None:  # pylint: disable=super-init-not-called
         self.h = cmd_channel  # type: FTPHandler
         self.cmd_channel = cmd_channel  # type: FTPHandler
-        self.hub  = cmd_channel.hub
+        self.hub: "SvcHub" = cmd_channel.hub
         self.args = cmd_channel.args
         self.uname = cmd_channel.uname
 
@@ -151,17 +155,17 @@ class FtpFs(AbstractedFS):
         self.listdirinfo = self.listdir
         self.chdir(".")
 
-    def log(self, msg , c   = 0)  :
+    def log(self, msg: str, c: Union[int, str] = 0) -> None:
         self.hub.log("ftpd", msg, c)
 
     def v2a(
         self,
-        vpath ,
-        r  = False,
-        w  = False,
-        m  = False,
-        d  = False,
-    )    :
+        vpath: str,
+        r: bool = False,
+        w: bool = False,
+        m: bool = False,
+        d: bool = False,
+    ) -> tuple[str, VFS, str]:
         try:
             vpath = vpath.replace("\\", "/").strip("/")
             rd, fn = os.path.split(vpath)
@@ -207,30 +211,30 @@ class FtpFs(AbstractedFS):
 
     def rv2a(
         self,
-        vpath ,
-        r  = False,
-        w  = False,
-        m  = False,
-        d  = False,
-    )    :
+        vpath: str,
+        r: bool = False,
+        w: bool = False,
+        m: bool = False,
+        d: bool = False,
+    ) -> tuple[str, VFS, str]:
         return self.v2a(join(self.cwd, vpath), r, w, m, d)
 
-    def ftp2fs(self, ftppath )  :
+    def ftp2fs(self, ftppath: str) -> str:
         # return self.v2a(ftppath)
         return ftppath  # self.cwd must be vpath
 
-    def fs2ftp(self, fspath )  :
+    def fs2ftp(self, fspath: str) -> str:
         # raise NotImplementedError()
         return fspath
 
-    def validpath(self, path )  :
+    def validpath(self, path: str) -> bool:
         if "/.hist/" in path:
             if "/up2k." in path or path.endswith("/dir.txt"):
                 raise FSE("Access to this file is forbidden", 1)
 
         return True
 
-    def open(self, filename , mode )  :
+    def open(self, filename: str, mode: str) -> typing.IO[Any]:
         r = "r" in mode
         w = "w" in mode or "a" in mode or "+" in mode
 
@@ -246,6 +250,7 @@ class FtpFs(AbstractedFS):
                 td = 0
 
         if w and need_unlink:
+            assert td  # type: ignore  # !rm
             if td >= -1 and td <= self.args.ftp_wt:
                 # within permitted timeframe; allow overwrite or resume
                 do_it = True
@@ -274,7 +279,7 @@ class FtpFs(AbstractedFS):
 
         return ret
 
-    def chdir(self, path )  :
+    def chdir(self, path: str) -> None:
         nwd = join(self.cwd, path)
         vfs, rem = self.hub.asrv.vfs.get(nwd, self.uname, False, False)
         if not vfs.realpath:
@@ -296,11 +301,11 @@ class FtpFs(AbstractedFS):
 
         self.cwd = nwd
 
-    def mkdir(self, path )  :
+    def mkdir(self, path: str) -> None:
         ap, vfs, _ = self.rv2a(path, w=True)
         bos.makedirs(ap, vf=vfs.flags)  # filezilla expects this
 
-    def listdir(self, path )  :
+    def listdir(self, path: str) -> list[str]:
         vpath = join(self.cwd, path)
         try:
             ap, vfs, rem = self.v2a(vpath, True, False)
@@ -346,7 +351,7 @@ class FtpFs(AbstractedFS):
             ret.sort()
             return ret
 
-    def rmdir(self, path )  :
+    def rmdir(self, path: str) -> None:
         ap = self.rv2a(path, d=True)[0]
         try:
             bos.rmdir(ap)
@@ -354,7 +359,7 @@ class FtpFs(AbstractedFS):
             if e.errno != errno.ENOENT:
                 raise
 
-    def remove(self, path )  :
+    def remove(self, path: str) -> None:
         if self.args.no_del:
             raise FSE("The delete feature is disabled in server config")
 
@@ -364,7 +369,7 @@ class FtpFs(AbstractedFS):
         except Exception as ex:
             raise FSE(str(ex))
 
-    def rename(self, src , dst )  :
+    def rename(self, src: str, dst: str) -> None:
         if self.args.no_mv:
             raise FSE("The rename/move feature is disabled in server config")
 
@@ -375,10 +380,10 @@ class FtpFs(AbstractedFS):
         except Exception as ex:
             raise FSE(str(ex))
 
-    def chmod(self, path , mode )  :
+    def chmod(self, path: str, mode: str) -> None:
         pass
 
-    def stat(self, path )  :
+    def stat(self, path: str) -> os.stat_result:
         try:
             ap = self.rv2a(path, r=True)[0]
             return bos.stat(ap)
@@ -393,15 +398,15 @@ class FtpFs(AbstractedFS):
 
             return st
 
-    def utime(self, path , timeval )  :
+    def utime(self, path: str, timeval: float) -> None:
         ap = self.rv2a(path, w=True)[0]
         bos.utime_c(logging.warning, ap, int(timeval), False)
 
-    def lstat(self, path )  :
+    def lstat(self, path: str) -> os.stat_result:
         ap = self.rv2a(path)[0]
         return bos.stat(ap)
 
-    def isfile(self, path )  :
+    def isfile(self, path: str) -> bool:
         try:
             st = self.stat(path)
             return stat.S_ISREG(st.st_mode)
@@ -411,11 +416,11 @@ class FtpFs(AbstractedFS):
 
             return False  # expected for mojibake in ftp_SIZE()
 
-    def islink(self, path )  :
+    def islink(self, path: str) -> bool:
         ap = self.rv2a(path)[0]
         return bos.path.islink(ap)
 
-    def isdir(self, path )  :
+    def isdir(self, path: str) -> bool:
         try:
             st = self.stat(path)
             return stat.S_ISDIR(st.st_mode)
@@ -425,37 +430,37 @@ class FtpFs(AbstractedFS):
 
             return True
 
-    def getsize(self, path )  :
+    def getsize(self, path: str) -> int:
         ap = self.rv2a(path)[0]
         return bos.path.getsize(ap)
 
-    def getmtime(self, path )  :
+    def getmtime(self, path: str) -> float:
         ap = self.rv2a(path)[0]
         return bos.path.getmtime(ap)
 
-    def realpath(self, path )  :
+    def realpath(self, path: str) -> str:
         return path
 
-    def lexists(self, path )  :
+    def lexists(self, path: str) -> bool:
         ap = self.rv2a(path)[0]
         return bos.path.lexists(ap)
 
-    def get_user_by_uid(self, uid )  :
+    def get_user_by_uid(self, uid: int) -> str:
         return "root"
 
-    def get_group_by_uid(self, gid )  :
+    def get_group_by_uid(self, gid: int) -> str:
         return "root"
 
 
 class FtpHandler(FTPHandler):
     abstracted_fs = FtpFs
-    #hub: "SvcHub"
-    #args: argparse.Namespace
-    #uname: str
+    hub: "SvcHub"
+    args: argparse.Namespace
+    uname: str
 
-    def __init__(self, conn , server , ioloop  = None)  :
-        self.hub  = FtpHandler.hub
-        self.args  = FtpHandler.args
+    def __init__(self, conn: Any, server: Any, ioloop: Any = None) -> None:
+        self.hub: "SvcHub" = FtpHandler.hub
+        self.args: argparse.Namespace = FtpHandler.args
         self.uname = "*"
 
         if PY2:
@@ -476,12 +481,12 @@ class FtpHandler(FTPHandler):
         self.cli_ip = cip
 
         # abspath->vpath mapping to resolve log_transfer paths
-        self.vfs_map   = {}
+        self.vfs_map: dict[str, str] = {}
 
         # reduce non-debug logging
         self.log_cmds_list = [x for x in self.log_cmds_list if x not in ("CWD", "XCWD")]
 
-    def ftp_STOR(self, file , mode  = "w")  :
+    def ftp_STOR(self, file: str, mode: str = "w") -> Any:
         # Optional[str]
         vp = join(self.fs.cwd, file).lstrip("/")
         try:
@@ -510,7 +515,7 @@ class FtpHandler(FTPHandler):
                 None,
             )
             t = hr.get("rejectmsg") or ""
-            if t or not hr:
+            if t or hr.get("rc") != 0:
                 if not t:
                     t = "Upload blocked by xbu server config: %r" % (vp,)
                 self.respond("550 %s" % (t,), logging.info)
@@ -523,13 +528,13 @@ class FtpHandler(FTPHandler):
 
     def log_transfer(
         self,
-        cmd ,
-        filename ,
-        receive ,
-        completed ,
-        elapsed ,
-        bytes ,
-    )  :
+        cmd: str,
+        filename: bytes,
+        receive: bool,
+        completed: bool,
+        elapsed: float,
+        bytes: int,
+    ) -> Any:
         # None
         ap = filename.decode("utf-8", "replace")
         vp = self.vfs_map.pop(ap, None)
@@ -565,7 +570,7 @@ except:
 
 
 class Ftpd(object):
-    def __init__(self, hub )  :
+    def __init__(self, hub: "SvcHub") -> None:
         self.hub = hub
         self.args = hub.args
 
@@ -637,6 +642,6 @@ class Ftpd(object):
         Daemon(ioloop.loop, "ftp")
 
 
-def join(p1 , p2 )  :
+def join(p1: str, p2: str) -> str:
     w = os.path.join(p1, p2.replace("\\", "/"))
     return os.path.normpath(w).replace("\\", "/")
