@@ -20,6 +20,12 @@ from datetime import datetime
 # print(currentframe().f_lineno)
 
 
+if True:  # pylint: disable=using-constant-test
+    from types import FrameType
+
+    import typing
+    from typing import Any, Optional, Union
+
 from .__init__ import ANYWIN, EXE, MACOS, PY2, TYPE_CHECKING, E, EnvParams, unicode
 from .authsrv import BAD_CFG, AuthSrv, derive_args, n_du_who, n_ver_who
 from .bos import bos
@@ -107,18 +113,18 @@ class SvcHub(object):
 
     def __init__(
         self,
-        args ,
-        dargs ,
-        argv ,
-        printed ,
-    )  :
+        args: argparse.Namespace,
+        dargs: argparse.Namespace,
+        argv: list[str],
+        printed: str,
+    ) -> None:
         self.args = args
         self.dargs = dargs
         self.argv = argv
-        self.E  = args.E
+        self.E: EnvParams = args.E
         self.no_ansi = args.no_ansi
         self.tz = UTC if args.log_utc else None
-        self.logf  = None
+        self.logf: Optional[typing.TextIO] = None
         self.logf_base_fn = ""
         self.is_dut = False  # running in unittest; always False
         self.stop_req = False
@@ -168,7 +174,7 @@ class SvcHub(object):
             raise Exception(BAD_CFG)
 
         # for non-http clients (ftp, tftp)
-        self.bans   = {}
+        self.bans: dict[str, int] = {}
         self.gpwd = Garda(self.args.ban_pw)
         self.gpwc = Garda(self.args.ban_pwc)
         self.g404 = Garda(self.args.ban_404)
@@ -405,13 +411,13 @@ class SvcHub(object):
         if args.ftp or args.ftps:
             from .ftpd import Ftpd
 
-            self.ftpd  = None
+            self.ftpd: Optional[Ftpd] = None
             zms += "f" if args.ftp else "F"
 
         if args.tftp:
             from .tftpd import Tftpd
 
-            self.tftpd  = None
+            self.tftpd: Optional[Tftpd] = None
 
         if args.ftp or args.ftps or args.tftp:
             Daemon(self.start_ftpd, "start_tftpd")
@@ -432,8 +438,8 @@ class SvcHub(object):
             args.zms = zms
 
         self.zc_ngen = 0
-        self.mdns  = None
-        self.ssdp  = None
+        self.mdns: Optional["MDNS"] = None
+        self.ssdp: Optional["SSDPd"] = None
 
         # decide which worker impl to use
         if self.check_mp_enable():
@@ -454,13 +460,13 @@ class SvcHub(object):
             for nm in args.ipr_u.values():
                 nm.mutex = threading.Lock()
 
-    def _db_onfail_ses(self)  :
+    def _db_onfail_ses(self) -> None:
         self.args.no_ses = True
 
-    def _db_onfail_idp(self)  :
+    def _db_onfail_idp(self) -> None:
         self.args.idp_store = 0
 
-    def setup_db(self, which )  :
+    def setup_db(self, which: str) -> None:
         """
         the "non-mission-critical" databases; if something looks broken then just nuke it
         """
@@ -497,6 +503,7 @@ class SvcHub(object):
             self.log("root", "WARNING: sqlite3 not available; %s" % (zs,), 3)
             return
 
+        assert sqlite3  # type: ignore  # !rm
 
         db_lock = db_path + ".lock"
         try:
@@ -552,7 +559,7 @@ class SvcHub(object):
                     pass
                 os.unlink(db_path)
 
-    def _create_session_db(self, cur )  :
+    def _create_session_db(self, cur: "sqlite3.Cursor") -> int:
         sch = [
             r"create table kv (k text, v int)",
             r"create table us (un text, si text, t0 int)",
@@ -567,7 +574,7 @@ class SvcHub(object):
         self.log("root", "created new sessions-db")
         return 1
 
-    def _create_idp_db(self, cur )  :
+    def _create_idp_db(self, cur: "sqlite3.Cursor") -> int:
         sch = [
             r"create table kv (k text, v int)",
             r"create table us (un text, gs text)",
@@ -582,14 +589,14 @@ class SvcHub(object):
 
     def _verify_db(
         self,
-        cur ,
-        which ,
-        pathopt ,
-        db_path ,
-        desc ,
-        sver ,
-        native_ver ,
-    )  :
+        cur: "sqlite3.Cursor",
+        which: str,
+        pathopt: str,
+        db_path: str,
+        desc: str,
+        sver: int,
+        native_ver: int,
+    ) -> str:
         # ensure writable (maybe owned by other user)
         db = cur.connection
 
@@ -631,13 +638,14 @@ class SvcHub(object):
         db.close()
         return ""
 
-    def setup_share_db(self)  :
+    def setup_share_db(self) -> None:
         al = self.args
         if not HAVE_SQLITE3:
             self.log("root", "sqlite3 not available; disabling --shr", 1)
             al.shr = ""
             return
 
+        assert sqlite3  # type: ignore  # !rm
 
         al.shr = al.shr.strip("/")
         if "/" in al.shr or not al.shr:
@@ -704,6 +712,8 @@ class SvcHub(object):
             r"insert into kv values ('sver', 2)",
         ]
 
+        assert db  # type: ignore  # !rm
+        assert cur  # type: ignore  # !rm
         if not sver:
             sver = VER_SHARES_DB
             for cmd in sch1 + sch2:
@@ -733,7 +743,7 @@ class SvcHub(object):
         cur.close()
         db.close()
 
-    def start_ftpd(self)  :
+    def start_ftpd(self) -> None:
         time.sleep(30)
 
         if hasattr(self, "ftpd") and not self.ftpd:
@@ -742,7 +752,7 @@ class SvcHub(object):
         if hasattr(self, "tftpd") and not self.tftpd:
             self.restart_tftpd()
 
-    def restart_ftpd(self)  :
+    def restart_ftpd(self) -> None:
         if not hasattr(self, "ftpd"):
             return
 
@@ -757,7 +767,7 @@ class SvcHub(object):
         self.ftpd = Ftpd(self)
         self.log("root", "started FTPd")
 
-    def restart_tftpd(self)  :
+    def restart_tftpd(self) -> None:
         if not hasattr(self, "tftpd"):
             return
 
@@ -768,7 +778,7 @@ class SvcHub(object):
 
         self.tftpd = Tftpd(self)
 
-    def thr_httpsrv_up(self)  :
+    def thr_httpsrv_up(self) -> None:
         time.sleep(1 if self.args.ign_ebind_all else 5)
         expected = self.broker.num_workers * self.tcpsrv.nsrv
         failed = expected - self.httpsrv_up
@@ -791,18 +801,18 @@ class SvcHub(object):
         self.retcode = 1
         self.sigterm()
 
-    def sigterm(self)  :
+    def sigterm(self) -> None:
         self.signal_handler(signal.SIGTERM, None)
 
-    def sticky_qr(self)  :
+    def sticky_qr(self) -> None:
         self._sticky_qr()
 
-    def _unsticky_qr(self, flush=True)  :
+    def _unsticky_qr(self, flush=True) -> None:
         print("\033[s\033[J\033[r\033[u", file=sys.stderr, end="")
         if flush:
             sys.stderr.flush()
 
-    def _sticky_qr(self, force  = False)  :
+    def _sticky_qr(self, force: bool = False) -> None:
         sz = termsize()
         if self.qr_tsz == sz:
             if not force:
@@ -872,7 +882,7 @@ class SvcHub(object):
                 break
             self._sticky_qr()
 
-    def cb_httpsrv_up(self)  :
+    def cb_httpsrv_up(self) -> None:
         self.httpsrv_up += 1
         if self.httpsrv_up != self.broker.num_workers:
             return
@@ -900,12 +910,12 @@ class SvcHub(object):
 
         self.after_httpsrv_up()
 
-    def after_httpsrv_up(self)  :
+    def after_httpsrv_up(self) -> None:
         self.up2k.init_vols()
 
         Daemon(self.sd_notify, "sd-notify")
 
-    def _feature_test(self)  :
+    def _feature_test(self) -> None:
         fok = []
         fng = []
         t_ff = "transcode audio, create spectrograms, video thumbnails"
@@ -959,7 +969,7 @@ class SvcHub(object):
         t += "\033[0m, see --deps (this is fine btw)"
         self.log("optional-dependencies", t, 6)
 
-    def _check_env(self)  :
+    def _check_env(self) -> None:
         al = self.args
 
         if self.args.no_bauth:
@@ -1010,7 +1020,7 @@ class SvcHub(object):
                 self.log("root", t2, 1)
                 raise Exception(t2)
 
-    def _process_config(self)  :
+    def _process_config(self) -> bool:
         al = self.args
 
         al.zm_on = al.zm_on or al.z_on
@@ -1071,7 +1081,7 @@ class SvcHub(object):
                 vs = os.path.expandvars(os.path.expanduser(vs))
                 setattr(al, k, vs)
 
-        for k in "idp_adm".split(" "):
+        for k in "idp_adm stats_u".split(" "):
             vs = getattr(al, k)
             vsa = [x.strip() for x in vs.split(",")]
             vsa = [x.lower() for x in vsa if x]
@@ -1193,14 +1203,14 @@ class SvcHub(object):
         derive_args(al)
         return True
 
-    def _ipa2re(self, txt)  :
+    def _ipa2re(self, txt) -> Optional[re.Pattern]:
         if txt in ("any", "0", ""):
             return None
 
         zs = txt.replace(" ", "").replace(".", "\\.").replace(",", "|")
         return re.compile("^(?:" + zs + ")")
 
-    def _setlimits(self)  :
+    def _setlimits(self) -> None:
         try:
             import resource
 
@@ -1240,7 +1250,7 @@ class SvcHub(object):
             self.log("root", t.format(soft, want, self.args.nc), 3)
             self.args.nc = min(self.args.nc, soft // 2)
 
-    def _logname(self)  :
+    def _logname(self) -> str:
         dt = datetime.now(self.tz)
         fn = str(self.args.lo)
         for fs in "YmdHMS":
@@ -1250,7 +1260,7 @@ class SvcHub(object):
 
         return fn
 
-    def _setup_logfile(self, printed )  :
+    def _setup_logfile(self, printed: str) -> None:
         base_fn = fn = sel_fn = self._logname()
         do_xz = fn.lower().endswith(".xz")
         if fn != self.args.lo:
@@ -1297,7 +1307,7 @@ class SvcHub(object):
         self.logf_base_fn = base_fn
         print(msg, end="")
 
-    def run(self)  :
+    def run(self) -> None:
         self.tcpsrv.run()
         if getattr(self.args, "z_chk", 0) and (
             getattr(self.args, "zm", False) or getattr(self.args, "zs", False)
@@ -1334,7 +1344,7 @@ class SvcHub(object):
         else:
             self.stop_thr()
 
-    def start_zeroconf(self)  :
+    def start_zeroconf(self) -> None:
         self.zc_ngen += 1
 
         if getattr(self.args, "zm", False):
@@ -1361,7 +1371,7 @@ class SvcHub(object):
             except:
                 self.log("root", "ssdp startup failed;\n" + min_ex(), 3)
 
-    def reload(self, rescan_all_vols , up2k )  :
+    def reload(self, rescan_all_vols: bool, up2k: bool) -> str:
         t = "config has been reloaded"
         with self.reload_mutex:
             self.log("root", "reloading config")
@@ -1375,12 +1385,12 @@ class SvcHub(object):
             self.broker.reload()
         return t
 
-    def _reload_sessions(self)  :
+    def _reload_sessions(self) -> None:
         with self.asrv.mutex:
             self.asrv.load_sessions(True)
         self.broker.reload_sessions()
 
-    def stop_thr(self)  :
+    def stop_thr(self) -> None:
         while not self.stop_req:
             with self.stop_cond:
                 self.stop_cond.wait(9001)
@@ -1391,7 +1401,7 @@ class SvcHub(object):
 
         self.shutdown()
 
-    def kill9(self, delay  = 0.0)  :
+    def kill9(self, delay: float = 0.0) -> None:
         if delay > 0.01:
             time.sleep(delay)
             print("component stuck; issuing sigkill")
@@ -1402,7 +1412,7 @@ class SvcHub(object):
         else:
             os.kill(os.getpid(), signal.SIGKILL)
 
-    def signal_handler(self, sig , frame )  :
+    def signal_handler(self, sig: int, frame: Optional[FrameType]) -> None:
         if self.stopping:
             if self.nsigs <= 0:
                 try:
@@ -1424,7 +1434,7 @@ class SvcHub(object):
         with self.stop_cond:
             self.stop_cond.notify_all()
 
-    def shutdown(self)  :
+    def shutdown(self) -> None:
         if self.stopping:
             return
 
@@ -1495,7 +1505,7 @@ class SvcHub(object):
             self.stopped = True
             sys.exit(ret)
 
-    def _log_disabled(self, src , msg , c   = 0)  :
+    def _log_disabled(self, src: str, msg: str, c: Union[int, str] = 0) -> None:
         if not self.logf:
             return
 
@@ -1529,7 +1539,7 @@ class SvcHub(object):
             if dt.day != self.cday or dt.month != self.cmon:
                 self._set_next_day(dt)
 
-    def _set_next_day(self, dt )  :
+    def _set_next_day(self, dt: datetime) -> None:
         if self.cday and self.logf and self.logf_base_fn != self._logname():
             self.logf.close()
             self._setup_logfile("")
@@ -1537,7 +1547,7 @@ class SvcHub(object):
         self.cday = dt.day
         self.cmon = dt.month
 
-    def _log_enabled(self, src , msg , c   = 0)  :
+    def _log_enabled(self, src: str, msg: str, c: Union[int, str] = 0) -> None:
         """handles logging from all components"""
         with self.log_mutex:
             dt = datetime.now(self.tz)
@@ -1594,7 +1604,7 @@ class SvcHub(object):
                 if not self.args.no_logflush:
                     self.logf.flush()
 
-    def pr(self, *a , **ka )  :
+    def pr(self, *a: Any, **ka: Any) -> None:
         try:
             with self.log_mutex:
                 print(*a, **ka)
@@ -1602,14 +1612,14 @@ class SvcHub(object):
             if ex.errno != errno.EPIPE:
                 raise
 
-    def check_mp_support(self)  :
+    def check_mp_support(self) -> str:
         if MACOS and not os.environ.get("PRTY_FORCE_MP"):
             return "multiprocessing is wonky on mac osx;"
         elif sys.version_info < (3, 3):
             return "need python 3.3 or newer for multiprocessing;"
 
         try:
-            x   = mp.Queue(1)
+            x: mp.Queue[tuple[str, str]] = mp.Queue(1)
             x.put(("foo", "bar"))
             if x.get()[0] != "foo":
                 raise Exception()
@@ -1618,7 +1628,7 @@ class SvcHub(object):
 
         return ""
 
-    def check_mp_enable(self)  :
+    def check_mp_enable(self) -> bool:
         if self.args.j == 1:
             return False
 
@@ -1644,7 +1654,7 @@ class SvcHub(object):
             self.log("svchub", "cannot efficiently use multiple CPU cores")
             return False
 
-    def sd_notify(self)  :
+    def sd_notify(self) -> None:
         try:
             zb = os.getenv("NOTIFY_SOCKET")
             if not zb:
@@ -1663,7 +1673,7 @@ class SvcHub(object):
         except:
             self.log("sd_notify", min_ex())
 
-    def log_stacks(self)  :
+    def log_stacks(self) -> None:
         td = time.time() - self.tstack
         if td < 300:
             self.log("stacks", "cooldown {}".format(td))
