@@ -35,6 +35,9 @@ from .util import (
     wunlink,
 )
 
+if True:  # pylint: disable=using-constant-test
+    from typing import Any, Optional, Union
+
 if TYPE_CHECKING:
     from .svchub import SvcHub
 
@@ -140,7 +143,7 @@ except:
 th_dir_cache = {}
 
 
-def thumb_path(histpath , rem , mtime , fmt , ffa )  :
+def thumb_path(histpath: str, rem: str, mtime: float, fmt: str, ffa: set[str]) -> str:
     # base16 = 16 = 256
     # b64-lc = 38 = 1444
     # base64 = 64 = 4096
@@ -179,7 +182,7 @@ def thumb_path(histpath , rem , mtime , fmt , ffa )  :
 
 
 class ThumbSrv(object):
-    def __init__(self, hub )  :
+    def __init__(self, hub: "SvcHub") -> None:
         self.hub = hub
         self.asrv = hub.asrv
         self.args = hub.args
@@ -188,9 +191,9 @@ class ThumbSrv(object):
         self.poke_cd = Cooldown(self.args.th_poke)
 
         self.mutex = threading.Lock()
-        self.busy   = {}
-        self.untemp   = {}
-        self.ram   = {}
+        self.busy: dict[str, list[threading.Condition]] = {}
+        self.untemp: dict[str, list[str]] = {}
+        self.ram: dict[str, float] = {}
         self.memcond = threading.Condition(self.mutex)
         self.stopping = False
         self.rm_nullthumbs = True  # forget failed conversions on startup
@@ -198,7 +201,7 @@ class ThumbSrv(object):
 
         self.exts_spec_unsafe = set(self.args.th_spec_cnv.split(","))
 
-        self.q     = Queue(self.nthr * 4)
+        self.q: Queue[Optional[tuple[str, str, str, VFS]]] = Queue(self.nthr * 4)
         for n in range(self.nthr):
             Daemon(self.worker, "thumb-{}-{}".format(n, self.nthr))
 
@@ -247,7 +250,7 @@ class ThumbSrv(object):
             for f in "avif avifs".split(" "):
                 self.fmt_pil.discard(f)
 
-        self.thumbable  = set()
+        self.thumbable: set[str] = set()
 
         if "pil" in self.args.th_dec:
             self.thumbable |= self.fmt_pil
@@ -262,24 +265,27 @@ class ThumbSrv(object):
             for zss in [self.fmt_ffi, self.fmt_ffv, self.fmt_ffa]:
                 self.thumbable |= zss
 
-    def log(self, msg , c   = 0)  :
+    def log(self, msg: str, c: Union[int, str] = 0) -> None:
         self.log_func("thumb", msg, c)
 
-    def shutdown(self)  :
+    def shutdown(self) -> None:
         self.stopping = True
+        Daemon(self._fire_sentinels, "thumbstopper")
+
+    def _fire_sentinels(self):
         for _ in range(self.nthr):
             self.q.put(None)
 
-    def stopped(self)  :
+    def stopped(self) -> bool:
         with self.mutex:
             return not self.nthr
 
-    def getres(self, vn , fmt )   :
+    def getres(self, vn: VFS, fmt: str) -> tuple[int, int]:
         mul = 3 if "3" in fmt else 1
         w, h = vn.flags["thsize"].split("x")
         return int(w) * mul, int(h) * mul
 
-    def get(self, ptop , rem , mtime , fmt )  :
+    def get(self, ptop: str, rem: str, mtime: float, fmt: str) -> Optional[str]:
         histpath = self.asrv.vfs.histtab.get(ptop)
         if not histpath:
             self.log("no histpath for %r" % (ptop,))
@@ -334,7 +340,7 @@ class ThumbSrv(object):
 
         return None
 
-    def getcfg(self)   :
+    def getcfg(self) -> dict[str, set[str]]:
         return {
             "thumbable": self.thumbable,
             "pil": self.fmt_pil,
@@ -345,7 +351,7 @@ class ThumbSrv(object):
             "ffa": self.fmt_ffa,
         }
 
-    def wait4ram(self, need , ttpath )  :
+    def wait4ram(self, need: float, ttpath: str) -> None:
         ram = self.args.th_ram_max
         if need > ram * 0.99:
             t = "file too big; need %.2f GiB RAM, but --th-ram-max is only %.1f"
@@ -362,7 +368,7 @@ class ThumbSrv(object):
                 # self.log("at RAM limit; used %.2f GiB, need %.2f more" % (used-need, need), 1)
                 self.memcond.wait(3)
 
-    def worker(self)  :
+    def worker(self) -> None:
         while not self.stopping:
             task = self.q.get()
             if not task:
@@ -433,7 +439,7 @@ class ThumbSrv(object):
                 except Exception as ex:
                     msg = "%s could not create thumbnail of %r\n%s"
                     msg = msg % (fun.__name__, abspath, min_ex())
-                    c   = 1 if "<Signals.SIG" in msg else "90"
+                    c: Union[str, int] = 1 if "<Signals.SIG" in msg else "90"
                     self.log(msg, c)
                     if getattr(ex, "returncode", 0) != 321:
                         if fun == funs[-1]:
@@ -488,7 +494,7 @@ class ThumbSrv(object):
         with self.mutex:
             self.nthr -= 1
 
-    def fancy_pillow(self, im , fmt , vn )  :
+    def fancy_pillow(self, im: "Image.Image", fmt: str, vn: VFS) -> "Image.Image":
         # exif_transpose is expensive (loads full image + unconditional copy)
         res = self.getres(vn, fmt)
         r = max(*res) * 2
@@ -515,7 +521,7 @@ class ThumbSrv(object):
 
         return im
 
-    def conv_image_pil(self, im , tpath , fmt , vn )  :
+    def conv_image_pil(self, im: "Image.Image", tpath: str, fmt: str, vn: VFS) -> None:
         try:
             im = self.fancy_pillow(im, fmt, vn)
         except Exception as ex:
@@ -543,12 +549,12 @@ class ThumbSrv(object):
 
         im.save(tpath, **args)
 
-    def conv_pil(self, abspath , tpath , fmt , vn )  :
+    def conv_pil(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         self.wait4ram(0.2, tpath)
         with Image.open(fsenc(abspath)) as im:
             self.conv_image_pil(im, tpath, fmt, vn)
 
-    def conv_vips(self, abspath , tpath , fmt , vn )  :
+    def conv_vips(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         self.wait4ram(0.2, tpath)
         crops = ["centre", "none"]
         if "f" in fmt:
@@ -566,9 +572,10 @@ class ThumbSrv(object):
                 if c == crops[-1]:
                     raise
 
+        assert img  # type: ignore  # !rm
         img.write_to_file(tpath, Q=40)
 
-    def conv_raw(self, abspath , tpath , fmt , vn )  :
+    def conv_raw(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         self.wait4ram(0.2, tpath)
         with rawpy.imread(abspath) as raw:
             thumb = raw.extract_thumb()
@@ -599,6 +606,7 @@ class ThumbSrv(object):
                     if c == crops[-1]:
                         raise
 
+            assert img  # type: ignore  # !rm
             img.write_to_file(tpath, Q=40)
         elif HAVE_PIL:
             if thumb.format == rawpy.ThumbFormat.BITMAP:
@@ -611,7 +619,7 @@ class ThumbSrv(object):
                 "either pil or vips is needed to process embedded bitmap thumbnails in raw files"
             )
 
-    def conv_ffmpeg(self, abspath , tpath , fmt , vn )  :
+    def conv_ffmpeg(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         self.wait4ram(0.2, tpath)
         ret, _, _, _ = ffprobe(abspath, int(vn.flags["convt"] / 2))
         if not ret:
@@ -619,7 +627,7 @@ class ThumbSrv(object):
 
         ext = abspath.rsplit(".")[-1].lower()
         if ext in ["h264", "h265"] or ext in self.fmt_ffi:
-            seek  = []
+            seek: list[bytes] = []
         else:
             dur = ret[".dur"][1] if ".dur" in ret else 4
             seek = [b"-ss", "{:.0f}".format(dur / 3).encode("utf-8")]
@@ -628,13 +636,13 @@ class ThumbSrv(object):
 
     def _ffmpeg_im(
         self,
-        abspath ,
-        tpath ,
-        fmt ,
-        vn ,
-        seek ,
-        imap ,
-    )  :
+        abspath: str,
+        tpath: str,
+        fmt: str,
+        vn: VFS,
+        seek: list[bytes],
+        imap: bytes,
+    ) -> None:
         scale = "scale={0}:{1}:force_original_aspect_ratio="
         if "f" in fmt:
             scale += "decrease,setsar=1:1"
@@ -676,13 +684,13 @@ class ThumbSrv(object):
         cmd += [fsenc(tpath)]
         self._run_ff(cmd, vn, "convt")
 
-    def _run_ff(self, cmd , vn , kto , oom  = 400)  :
+    def _run_ff(self, cmd: list[bytes], vn: VFS, kto: str, oom: int = 400) -> None:
         # self.log((b" ".join(cmd)).decode("utf-8"))
         ret, _, serr = runcmd(cmd, timeout=vn.flags[kto], nice=True, oom=oom)
         if not ret:
             return
 
-        c   = "90"
+        c: Union[str, int] = "90"
         t = "FFmpeg failed (probably a corrupt file):\n"
         if (
             (not self.args.th_ff_jpg or time.time() - int(self.args.th_ff_jpg) < 60)
@@ -721,7 +729,7 @@ class ThumbSrv(object):
         self.log(t + txt, c=c)
         raise sp.CalledProcessError(ret, (cmd[0], b"...", cmd[-1]))
 
-    def conv_waves(self, abspath , tpath , fmt , vn )  :
+    def conv_waves(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         ret, _, _, _ = ffprobe(abspath, int(vn.flags["convt"] / 2))
         if "ac" not in ret:
             raise Exception("not audio")
@@ -782,14 +790,14 @@ class ThumbSrv(object):
                 atomic_move(self.log, wtpath, tpath, vn.flags)
 
     def conv_emb_cv(
-        self, abspath , tpath , fmt , vn , strm  
-    )  :
+        self, abspath: str, tpath: str, fmt: str, vn: VFS, strm: dict[str, Any]
+    ) -> None:
         self.wait4ram(0.2, tpath)
         self._ffmpeg_im(
             abspath, tpath, fmt, vn, [], b"0:" + strm["index"].encode("ascii")
         )
 
-    def conv_spec(self, abspath , tpath , fmt , vn )  :
+    def conv_spec(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         ret, raw, strms, ctnr = ffprobe(abspath, int(vn.flags["convt"] / 2))
         if "ac" not in ret:
             raise Exception("not audio")
@@ -885,7 +893,7 @@ class ThumbSrv(object):
         cmd += [fsenc(tpath)]
         self._run_ff(cmd, vn, "convt")
 
-    def conv_mp3(self, abspath , tpath , fmt , vn )  :
+    def conv_mp3(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         quality = self.args.q_mp3.lower()
         if self.args.no_acode or not quality:
             raise Exception("disabled in server config")
@@ -924,7 +932,7 @@ class ThumbSrv(object):
         # fmt: on
         self._run_ff(cmd, vn, "aconvt", oom=300)
 
-    def conv_flac(self, abspath , tpath , fmt , vn )  :
+    def conv_flac(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         if self.args.no_acode or not self.args.allow_flac:
             raise Exception("flac not permitted in server config")
 
@@ -949,7 +957,7 @@ class ThumbSrv(object):
         # fmt: on
         self._run_ff(cmd, vn, "aconvt", oom=300)
 
-    def conv_wav(self, abspath , tpath , fmt , vn )  :
+    def conv_wav(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         if self.args.no_acode or not self.args.allow_wav:
             raise Exception("wav not permitted in server config")
 
@@ -984,7 +992,7 @@ class ThumbSrv(object):
         # fmt: on
         self._run_ff(cmd, vn, "aconvt", oom=300)
 
-    def conv_opus(self, abspath , tpath , fmt , vn )  :
+    def conv_opus(self, abspath: str, tpath: str, fmt: str, vn: VFS) -> None:
         if self.args.no_acode or not self.args.q_opus:
             raise Exception("disabled in server config")
 
@@ -1006,14 +1014,14 @@ class ThumbSrv(object):
 
     def _conv_owa(
         self,
-        abspath ,
-        tpath ,
-        tags   ,
-        rawtags  ,
-        enc ,
-        bq ,
-        vn ,
-    )  :
+        abspath: str,
+        tpath: str,
+        tags: dict[str, tuple[int, Any]],
+        rawtags: dict[str, list[Any]],
+        enc: str,
+        bq: bytes,
+        vn: VFS,
+    ) -> None:
         if tpath.endswith(".owa"):
             container = b"webm"
             tagset = [b"-map_metadata", b"-1"]
@@ -1042,14 +1050,14 @@ class ThumbSrv(object):
 
     def _conv_caf(
         self,
-        abspath ,
-        tpath ,
-        tags   ,
-        rawtags  ,
-        enc ,
-        bq ,
-        vn ,
-    )  :
+        abspath: str,
+        tpath: str,
+        tags: dict[str, tuple[int, Any]],
+        rawtags: dict[str, list[Any]],
+        enc: str,
+        bq: bytes,
+        vn: VFS,
+    ) -> None:
         tmp_opus = tpath + ".opus"
         try:
             wunlink(self.log, tmp_opus, vn.flags)
@@ -1131,7 +1139,7 @@ class ThumbSrv(object):
         except:
             pass
 
-    def big_tags(self, raw_tags  )  :
+    def big_tags(self, raw_tags: dict[str, list[str]]) -> list[bytes]:
         ret = []
         for k, vs in raw_tags.items():
             for v in vs:
@@ -1141,7 +1149,7 @@ class ThumbSrv(object):
                     break
         return ret
 
-    def poke(self, tdir )  :
+    def poke(self, tdir: str) -> None:
         if not self.poke_cd.poke(tdir):
             return
 
@@ -1153,7 +1161,7 @@ class ThumbSrv(object):
         except:
             pass
 
-    def cleaner(self)  :
+    def cleaner(self) -> None:
         interval = self.args.th_clean
         while True:
             ndirs = 0
@@ -1172,7 +1180,7 @@ class ThumbSrv(object):
             self.rm_nullthumbs = False
             time.sleep(interval)
 
-    def clean(self, histpath )  :
+    def clean(self, histpath: str) -> int:
         ret = 0
         for cat in ["th", "ac"]:
             top = os.path.join(histpath, cat)
@@ -1183,7 +1191,7 @@ class ThumbSrv(object):
 
         return ret
 
-    def _clean(self, cat , thumbpath )  :
+    def _clean(self, cat: str, thumbpath: str) -> int:
         # self.log("cln {}".format(thumbpath))
         exts = EXTS_TH if cat == "th" else EXTS_AC
         maxage = getattr(self.args, cat + "_maxage")
